package com.genericparallel;

import java.util.Map;

import javax.script.Bindings;
import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;

import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.w3c.dom.Node;

import com.bea.wli.config.Ref;
import com.bea.wli.config.component.NotFoundException;
import com.bea.wli.sb.resources.config.XmlEntryDocument;
import com.bea.wli.sb.resources.xml.XmlRepository;
import com.bea.wli.sb.resources.xquery.XqueryRepository;

public class Javascript {

	/**
	 * Execute a script with the body of JSON and return the result.
	 * @param json JSON data as string
	 * @param scriptPath Path to XML containing Javascript under OSB Projects (e.g. 'Foo/Bar/InsertCredentials.xml') 
	 * @param xml Other parameters for Javascript
	 * @return New JSON value.
	 */
	public static String execute(String json, String scriptPath, XmlObject xml) throws Exception {
	    // load JS from path
	    XmlObject scriptXml = readXml(scriptPath);
	    
	    XmlCursor cursor = scriptXml.newCursor();
	    cursor.toFirstChild();
	    String script = cursor.getTextValue();

	    return executeInner(json,script,xml);
	}	
	    		
	private static String executeInner(String json, String script, XmlObject xml) throws Exception {
	    ScriptEngineManager manager = new ScriptEngineManager();
	    ScriptEngine engine = manager.getEngineByName("javascript");

	    // collect parameters
	    Bindings bindings = engine.createBindings();
	    if( xml != null ) {
	    	XmlObject[] params = xml.execQuery("//var");
	    	for (XmlObject p : params) {
	    		String name = null;
	    		String value = null;

	    	    XmlCursor cursor = p.newCursor();
	    	    cursor.toFirstChild();
	    	    cursor.toFirstChild();

	    	    String varName = cursor.getName().getLocalPart();
				if( "name".equals(varName) ) {
					name = cursor.getTextValue();
				}
				if( "value".equals(varName) ) {
					value = cursor.getTextValue();
				}
				
				cursor.toNextSibling();

	    	    varName = cursor.getName().getLocalPart();
				if( "name".equals(varName) ) {
					name = cursor.getTextValue();
				}
				if( "value".equals(varName) ) {
					value = cursor.getTextValue();
				}
				
				if( name != null ) {
					System.out.println("Adding: "+name+"="+value);
					bindings.put(name, value);
				}
			}
	    }

	    addJSON(engine,bindings);
	    
	    // inject and parse the main value
	    Object jo = engine.eval("var json = "+json+";",bindings);
	    
	    // execute the script
	    engine.eval(script,bindings);
	    
	    // return the value of output variable as String
	    return (String)engine.eval("JSON.stringify(output)",bindings);
	}
	
    private static void addJSON(ScriptEngine engine, Bindings bindings) throws ScriptException {
    	engine.eval(
    			"JSON = {\n"+
    			"    parse: function(sJSON) { return eval('(' + sJSON + ')'); },\n"+
    			"    stringify: (function () {\n"+
    			"      var toString = Object.prototype.toString;\n"+
    			"      var isArray = Array.isArray || function (a) { return toString.call(a) === '[object Array]'; };\n"+
    			"      var escMap = {'\\\"': '\\\\\"', '\\\\': '\\\\\\\\', '\\b': '\\\\b', '\\f': '\\\\f', '\\n': '\\\\n', '\\r': '\\\\r', '\\t': '\\\\t'};\n"+
    			"      var escMap = {};\n"+
    			"      var escFunc = function (m) { return escMap[m] || '\\u' + (m.charCodeAt(0) + 0x10000).toString(16).substr(1); };\n"+
    			"      var escRE = /[\\\"\\u0000-\\u001F\\u2028\\u2029]/g;\n"+
    			"      return function stringify(value) {\n"+
    			"        if (value == null) {\n"+
    			"          return 'null';\n"+
    			"        } else if (typeof value === 'number') {\n"+
    			"          return isFinite(value) ? value.toString() : 'null';\n"+
    			"        } else if (typeof value === 'boolean') {\n"+
    			"          return value.toString();\n"+
    			"        } else if (typeof value === 'object') {\n"+
    			"          if (typeof value.toJSON === 'function') {\n"+
    			"            return stringify(value.toJSON());\n"+
    			"          } else if (isArray(value)) {\n"+
    			"            var res = '[';\n"+
    			"            for (var i = 0; i < value.length; i++)\n"+
    			"              res += (i ? ', ' : '') + stringify(value[i]);\n"+
    			"            return res + ']';\n"+
    			"          } else if (toString.call(value) === '[object Object]') {\n"+
    			"            var tmp = [];\n"+
    			"            for (var k in value) {\n"+
    			"              if (value.hasOwnProperty(k))\n"+
    			"                tmp.push(stringify(k) + ': ' + stringify(value[k]));\n"+
    			"            }\n"+
    			"            return '{' + tmp.join(', ') + '}';\n"+
    			"          }\n"+
    			"        }\n"+
    			"        return '\"' + value.toString().replace(escRE, escFunc) + '\"';\n"+
    			"      };\n"+
    			"    })()\n"+
    			"  };",bindings);
	}

	private static XqueryRepository xqr = XqueryRepository.get();

    /**
	 * Reads XML resource as XmlObject
	 *  
	 * @param xmlRefPath Project path e.g. "Foo/XML/Data"
	 * @return XmlObject with parsed XML
	 */
    private static XmlObject readXml(String xmlRefPath) {
    	Ref ref = new Ref("XML", Ref.getNames(xmlRefPath));
        XmlObject xmlObject = null;
        try {
            XmlEntryDocument xmlEntryDocument = XmlRepository.get().getEntry(ref);
            String xmlContent = xmlEntryDocument.getXmlEntry().getXmlContent();
            xmlObject = XmlObject.Factory.parse(xmlContent);
        } catch (NotFoundException e) {
            throw new RuntimeException("XML Resource not found: "+xmlRefPath);
        } catch (XmlException e) {
            throw new RuntimeException("Error parsing XML content: "+xmlRefPath, e);
        }
        return xmlObject;
    }
}